import Graph from 'graphology';
import AbstractGraph, { DirectedGraph, MultiGraph } from 'graphology';
import { useSchema } from '..';
import reducer, { selectSchema, setSchema, initialState } from './schemaSlice';
// import { deleteBook, updateBook, addNewBook } from '../redux/bookSlice';
import { store } from './store';

import {
  movieSchema,
  northWindSchema,
  simpleSchema,
  twitterSchema,
} from '@graphpolaris/shared/mock-data';

describe('SchemaSlice Tests', () => {
  it('should make a graphology graph', () => {
    const graph = new MultiGraph({ allowSelfLoops: true });
    expect(graph);

    const graph2 = new MultiGraph();
    expect(graph2);

    const exported = graph.export();
    expect(exported);
  });

  it('export and reimport equality check for graphology graph', () => {
    const graph = new MultiGraph({ allowSelfLoops: true });
    expect(graph);
    const exported = graph.export();
    expect(exported);

    const graphReloaded = MultiGraph.from(exported);
    expect(graphReloaded).toStrictEqual(graph);
  });

  it('get the initial state', () => {
    expect(initialState);
  });

  it('state should return the initial state', () => {
    let state = store.getState();

    const schema = state.schema;
    expect(schema);

    const graph = MultiGraph.from(schema.graphologySerialized);

    // console.log(graph);
    // console.log(initialState);
    expect(graph);
  });
});

test.each([
  { data: simpleSchema },
  { data: movieSchema },
  { data: northWindSchema },
  { data: twitterSchema },
])('store and retrieve', ({ data }) => {

  let state = store.getState();

  const schema = state.schema;
  expect(schema);

  expect(data).toBeDefined();
});
