import { Layout } from './Layout';
import { ILayoutFactory, LayoutAlgorithm } from './layout-creator-usecase';

export type CytoscapeProvider = 'Cytoscape';

export type CytoscapeLayoutAlgorithms =
  | `${CytoscapeProvider}_coupe`
  | `${CytoscapeProvider}_i4`;

/**
 * This is a ConcreteCreator
 */
export class CytoscapeFactory
  implements ILayoutFactory<CytoscapeLayoutAlgorithms>
{
  createLayout(LayoutAlgorithm: CytoscapeLayoutAlgorithms): Cytoscape | null {
    switch (LayoutAlgorithm) {
      case 'Cytoscape_coupe':
        return new CytoscapeCoupe();
      case 'Cytoscape_i4':
        return new CytoscapeI4();
      default:
        return null;
    }
  }
}

export abstract class Cytoscape extends Layout<CytoscapeProvider> {
  constructor(public override algorithm: LayoutAlgorithm<CytoscapeProvider>) {
    super('Cytoscape', algorithm);
  }

  public specialCytoscapeFunction() {
    console.log('Only Cytoscape Layouts can do this.');
  }
}

/**
 * This is a ConcreteProduct
 */
class CytoscapeI4 extends Cytoscape {
  constructor() {
    super('Cytoscape_i4');
  }
}

/**
 * This is a ConcreteProduct
 */
class CytoscapeCoupe extends Cytoscape {
  constructor() {
    super('Cytoscape_coupe');
  }
}
