import {
  AllLayoutAlgorithms,
  LayoutFactory,
} from '@graphpolaris/shared/graph-layout';

import {
  movieSchema,
  northWindSchema,
  simpleSchema,
  twitterSchema,
} from '@graphpolaris/shared/mock-data';
import Graph, { MultiGraph } from 'graphology';

import ladder from 'graphology-generators/classic/ladder';

/**
 * @jest-environment jsdom
 */
describe('LayoutFactory', () => {
  /**
   * @jest-environment jsdom
   */
  it('should work with noverlap from graphology ', () => {
    const graph = new MultiGraph();

    // Adding some nodes
    graph.addNode('John', { x: 0, y: 0 });
    graph.addNode('Martha', { x: 0, y: 0 });

    // Adding an edge
    graph.addEdge('John', 'Martha');

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_noverlap');

    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      console.log(node, attr);
    });
  });

  test('should work with noverlap from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 10);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_noverlap');
    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      console.log(node, attr);
    });
  });

  test('should work with random from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 10);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_random');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      console.log(node, attr);
    });
  });

  test('should work with circular from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 100);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();

    const layout = layoutFactory.createLayout('Graphology_circular');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      console.log(node, attr);
    });
  });

  test('should work with circular from graphology on generated graph', () => {
    // Creating a ladder graph
    const graph = ladder(Graph, 100);

    graph.forEachNode((node, attr) => {
      graph.setNodeAttribute(node, 'x', 0);
      graph.setNodeAttribute(node, 'y', 0);
    });

    const layoutFactory = new LayoutFactory();
    const layout = layoutFactory.createLayout('Graphology_forceAtlas2');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      console.log(node, attr);
    });
  });

  test('should add x,y for graphology layouts if not existing', () => {
    // console.log(Object.keys(AllLayoutAlgorithms))

    const graph = ladder(Graph, 100);

    graph.forEachNode((node, attr) => {
      expect(graph.getNodeAttribute(node, 'x')).toBeUndefined();
      expect(graph.getNodeAttribute(node, 'y')).toBeUndefined();
    });

    const layoutFactory = new LayoutFactory();
    const layout = layoutFactory.createLayout('Graphology_forceAtlas2');
    layout?.setDimensions(100, 100);
    layout?.layout(graph);

    graph.forEachNode((node, attr) => {
      expect(graph.getNodeAttribute(node, 'x')).toBeDefined();
      expect(graph.getNodeAttribute(node, 'y')).toBeDefined();
    });
  });
});
